-- =====================================================================
-- Copyright © 2010-2012 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- =====================================================================

-- Possible generics values: 
--		HS = {HASH_SIZE_256, HASH_SIZE_512} 	
--      UF  = {1, 2} (Unrolling factor)
--      PPL = {2, 4} 

-- Supported architectures :


library ieee;
use ieee.std_logic_1164.all; 
use work.sha3_pkg.all;
use work.sha3_jh_package.all;

entity jh_top_ppl is		
	generic (		
		PPL : integer := 2;
		UF : integer := 1;
		HS : integer := HASH_SIZE_256
	); 
	port (		
		-- global
		rst 	: in std_logic;
		clk 	: in std_logic;
		
		--fifo
		src_ready : in std_logic_vector(ppl-1 downto 0);
		src_read  : out std_logic_vector(ppl-1 downto 0);
		dst_ready : in std_logic;
		dst_write : out std_logic;		
		din		: in std_logic_vector(ppl*w-1 downto 0);
		dout	: out std_logic_vector(w-1 downto 0)
	);	   
end jh_top_ppl;


architecture structure of jh_top_ppl is 
	constant rc_mode : integer := MEMORY;
	
	-- fsm1
	signal ein, lc, ec: std_logic_vector(ppl-1 downto 0);
	signal zc0, final_segment : std_logic_vector(ppl-1 downto 0);
	-- fsm2
	signal emsg : std_logic_vector(ppl-1 downto 0);
	signal schain, er, sf : std_logic;   
	signal smsg : std_logic_vector(log2(ppl)-1 downto 0);
	signal round : std_logic_vector(5-(UF-1) downto 0);
	signal sout : std_logic_vector(log2(ppl)-1 downto 0);	
	signal sid : std_logic;
	-- fsm3
	signal shift_dout : std_logic_vector(ppl-1 downto 0);
	signal load_id, load_hash : std_logic_vector(ppl-1 downto 0);
	-- top fsm									 
begin			
	fseg : for i in 0 to ppl-1 generate
		final_segment(i) <= din(w*ppl-i*w-1);	-- MSB of a word
	end generate; 
	
	control_gen : entity work.jh_control_mem_ppl(struct)
		generic map ( ppl => ppl, HS => HS, UF => UF )
		port map (
		rst => rst, clk => clk,
		src_ready => src_ready, src_read => src_read, dst_ready => dst_ready, dst_write => dst_write,	  
		zc0 => zc0, final_segment => final_segment, ein => ein, ec => ec, lc => lc, smsg => smsg,
		er => er,  emsg => emsg, sf => sf, schain => schain, round => round,
		shift_dout => shift_dout, load_id => load_id, load_hash => load_hash,  sout => sout,  sid => sid
	);			
	
	datapath_gen : entity work.jh_datapath_mem_ppl(struct) 
		generic map ( ppl => ppl, HS => HS, UF => UF )
		port map (
		rst => rst, clk => clk, din => din, dout => dout,
		zc0 => zc0, ein => ein, ec => ec, lc => lc, smsg => smsg, 
		er => er, emsg => emsg, sf => sf, round => round, schain => schain, 
		shift_dout => shift_dout, load_id => load_id, load_hash => load_hash, sout => sout, sid => sid
	);		  

end structure;
	
	